@@ -2,18 +2,19 @@
 # by importing them here in conftest.py they are discoverable by py.test
 # no matter how it is invoked within the source tree.
 
-from astropy.tests.pytest_plugins import *
+import os
 
+from astropy.tests.pytest_plugins import *
 from .tests.pytest_fixtures import *
 
 # This is to figure out ccdproc version, rather than using Astropy's
-from . import version
-
 try:
-    packagename = os.path.basename(os.path.dirname(__file__))
-    TESTED_VERSIONS[packagename] = version.version
-except NameError:
-    pass
+    from .version import version
+except ImportError:
+    version = 'dev'
+
+packagename = os.path.basename(os.path.dirname(__file__))
+TESTED_VERSIONS[packagename] = version
 
 ## Uncomment the following line to treat all DeprecationWarnings as
 ## exceptions
@@ -26,5 +27,5 @@ try:
     PYTEST_HEADER_MODULES['astroscrappy'] = 'astroscrappy'
     PYTEST_HEADER_MODULES['reproject'] = 'reproject'
     del PYTEST_HEADER_MODULES['h5py']
-except NameError:
+except KeyError:
     pass
