@@ -144,7 +144,7 @@ def test_combiner_sigmaclip_high():
                 CCDData(np.zeros((10, 10)) + 1000, unit=u.adu)]
 
     c = Combiner(ccd_list)
-    #using mad for more rubust statistics vs. std
+    #using mad for more robust statistics vs. std
     c.sigma_clipping(high_thresh=3, low_thresh=None, func=np.median,
                      dev_func=mad)
     assert c.data_arr[5].mask.all()
@@ -179,7 +179,7 @@ def test_combiner_sigmaclip_low():
                 CCDData(np.zeros((10, 10)) - 1000, unit=u.adu)]
 
     c = Combiner(ccd_list)
-    #using mad for more rubust statistics vs. std
+    #using mad for more robust statistics vs. std
     c.sigma_clipping(high_thresh=None, low_thresh=3, func=np.median,
                      dev_func=mad)
     assert c.data_arr[5].mask.all()
@@ -383,3 +383,132 @@ def test_combiner_uncertainty_average_mask():
     ref_uncertainty[5, 5] = np.std([2, 3]) / np.sqrt(2)
     np.testing.assert_array_almost_equal(ccd.uncertainty.array,
                                          ref_uncertainty)
+
+def test_combiner_3d():
+    data1 = CCDData(3 * np.ones((5,5,5)), unit=u.adu)
+    data2 = CCDData(2 * np.ones((5,5,5)), unit=u.adu)
+    data3 = CCDData(4 * np.ones((5,5,5)), unit=u.adu)
+
+    ccd_list = [data1, data2, data3]
+
+    c = Combiner(ccd_list)
+    assert c.data_arr.shape == (3, 5, 5, 5)
+    assert c.data_arr.mask.shape == (3, 5, 5, 5)
+
+    ccd = c.average_combine()
+    assert ccd.shape == (5, 5, 5)
+    np.testing.assert_array_almost_equal(ccd.data, data1, decimal = 4)
+
+def test_3d_combiner_with_scaling(ccd_data):
+    # The factors below are not particularly important; just avoid anything
+    # whose average is 1.
+    ccd_data = CCDData(np.ones((5,5,5)), unit=u.adu)
+    ccd_data_lower = CCDData(3 * np.ones((5,5,5)), unit=u.adu)
+    ccd_data_higher = CCDData(0.9 * np.ones((5,5,5)), unit=u.adu)
+    combiner = Combiner([ccd_data, ccd_data_higher, ccd_data_lower])
+    # scale each array to the mean of the first image
+    scale_by_mean = lambda x: ccd_data.data.mean()/np.ma.average(x)
+    combiner.scaling = scale_by_mean
+    avg_ccd = combiner.average_combine()
+    # Does the mean of the scaled arrays match the value to which it was
+    # scaled?
+    np.testing.assert_almost_equal(avg_ccd.data.mean(),
+                                   ccd_data.data.mean())
+    assert avg_ccd.shape == ccd_data.shape
+    median_ccd = combiner.median_combine()
+    # Does median also scale to the correct value?
+    np.testing.assert_almost_equal(np.median(median_ccd),
+                                   np.median(ccd_data.data))
+
+    # Set the scaling manually...
+    combiner.scaling = [scale_by_mean(combiner.data_arr[i]) for i in range(3)]
+    avg_ccd = combiner.average_combine()
+    np.testing.assert_almost_equal(avg_ccd.data.mean(),
+                                   ccd_data.data.mean())
+    assert avg_ccd.shape == ccd_data.shape
+
+
+def test_clip_extrema_3d():
+    ccdlist = [CCDData(np.ones((3, 3, 3))*90., unit="adu"),\
+               CCDData(np.ones((3, 3, 3))*20., unit="adu"),\
+               CCDData(np.ones((3, 3, 3))*10., unit="adu"),\
+               CCDData(np.ones((3, 3, 3))*40., unit="adu"),\
+               CCDData(np.ones((3, 3, 3))*25., unit="adu"),\
+               CCDData(np.ones((3, 3, 3))*35., unit="adu"),\
+              ]
+    c = Combiner(ccdlist)
+    c.clip_extrema(nlow=1, nhigh=1)
+    result = c.average_combine()
+    expected = CCDData(np.ones((3, 3, 3)) * 30, unit="adu")
+    np.testing.assert_array_equal(result, expected)
+
+
+@pytest.mark.parametrize('comb_func', ['average_combine', 'median_combine'])
+def test_writeable_after_combine(ccd_data, tmpdir, comb_func):
+    tmp_file = tmpdir.join('tmp.fits')
+    from ..combiner import Combiner
+    combined = Combiner([ccd_data for _ in range(3)])
+    ccd2 = getattr(combined, comb_func)()
+    # This should not fail because the resulting uncertainty has a mask
+    ccd2.write(tmp_file.strpath)
+
+def test_clip_extrema():
+    ccdlist = [CCDData(np.ones((3, 5))*90., unit="adu"),\
+               CCDData(np.ones((3, 5))*20., unit="adu"),\
+               CCDData(np.ones((3, 5))*10., unit="adu"),\
+               CCDData(np.ones((3, 5))*40., unit="adu"),\
+               CCDData(np.ones((3, 5))*25., unit="adu"),\
+               CCDData(np.ones((3, 5))*35., unit="adu"),\
+              ]
+    ccdlist[0].data[0,1] = 3.1
+    ccdlist[1].data[1,2] = 100.1
+    ccdlist[1].data[2,0] = 100.1
+    c = Combiner(ccdlist)
+    c.clip_extrema(nlow=1, nhigh=1)
+    result = c.average_combine()
+    expected = [[ 30.,  22.5, 30. , 30., 30.],
+                [ 30. , 30. , 47.5, 30., 30.],
+                [ 47.5, 30. , 30. , 30., 30.]]
+    np.testing.assert_array_equal(result, expected)
+
+def test_clip_extrema_via_combine():
+    ccdlist = [CCDData(np.ones((3, 5))*90., unit="adu"),\
+               CCDData(np.ones((3, 5))*20., unit="adu"),\
+               CCDData(np.ones((3, 5))*10., unit="adu"),\
+               CCDData(np.ones((3, 5))*40., unit="adu"),\
+               CCDData(np.ones((3, 5))*25., unit="adu"),\
+               CCDData(np.ones((3, 5))*35., unit="adu"),\
+              ]
+    ccdlist[0].data[0,1] = 3.1
+    ccdlist[1].data[1,2] = 100.1
+    ccdlist[1].data[2,0] = 100.1
+    result = combine(ccdlist, clip_extrema=True, nlow=1, nhigh=1,)
+    expected = [[ 30.,  22.5, 30. , 30., 30.],
+                [ 30. , 30. , 47.5, 30., 30.],
+                [ 47.5, 30. , 30. , 30., 30.]]
+    np.testing.assert_array_equal(result, expected)
+
+
+def test_clip_extrema_with_other_rejection():
+    ccdlist = [CCDData(np.ones((3, 5))*90., unit="adu"),\
+               CCDData(np.ones((3, 5))*20., unit="adu"),\
+               CCDData(np.ones((3, 5))*10., unit="adu"),\
+               CCDData(np.ones((3, 5))*40., unit="adu"),\
+               CCDData(np.ones((3, 5))*25., unit="adu"),\
+               CCDData(np.ones((3, 5))*35., unit="adu"),\
+              ]
+    ccdlist[0].data[0,1] = 3.1
+    ccdlist[1].data[1,2] = 100.1
+    ccdlist[1].data[2,0] = 100.1
+    c = Combiner(ccdlist)
+    ## Reject ccdlist[1].data[1,2] by other means
+    c.data_arr.mask[1,1,2] = True
+    ## Reject ccdlist[1].data[1,2] by other means
+    c.data_arr.mask[3,0,0] = True
+
+    c.clip_extrema(nlow=1, nhigh=1)
+    result = c.average_combine()
+    expected = [[ 80./3.,  22.5, 30. , 30., 30.],
+                [ 30. , 30. , 47.5, 30., 30.],
+                [ 47.5, 30. , 30. , 30., 30.]]
+    np.testing.assert_array_equal(result, expected)
